@@ -105,6 +105,10 @@ ncclCommDestroy_impl(ncclComm_t comm);
 ncclResult_t
 ncclCommAbort_impl(ncclComm_t comm);
 
+ncclResult_t
+ncclCommShrink_impl(ncclComm_t comm, int* excludeRanksList, int excludeRanksCount, ncclComm_t *newcomm,
+                    ncclConfig_t* config, int shrinkFlags);
+
 ncclResult_t
 ncclCommSplit_impl(ncclComm_t comm, int color, int key, ncclComm_t* newcomm,
                    ncclConfig_t* config);
@@ -153,6 +157,17 @@ ncclCommRegister_impl(const ncclComm_t comm, void* buff, size_t size, void** han
 ncclResult_t
 ncclCommDeregister_impl(const ncclComm_t comm, void* handle);
 
+ncclResult_t
+ncclCommWindowRegister_impl(ncclComm_t comm, void* buff, size_t size, ncclWindow_t* win, int winFlags);
+
+ncclResult_t
+ncclCommWindowDeregister_impl(ncclComm_t comm, ncclWindow_t win);
+
+ncclResult_t
+ncclAllReduceWithBias_impl(const void* sendbuff, void* recvbuff, size_t count,
+                   ncclDataType_t datatype, ncclRedOp_t op, ncclComm* comm,
+                   cudaStream_t stream, const void* acc);
+
 namespace rccl
 {
 namespace
@@ -174,6 +189,7 @@ compute_table_size(size_t nmembers)
     static_assert(offsetof(TABLE, MEMBER) == compute_table_offset(IDX),                  \
                   "Do not re-arrange the table members")
 
+// DO NOT REORDER, ADD NEW ITEMS TO BOTTOM
 RCCL_ASSERT_OFFSET(rcclApiFuncTable, ncclAllGather_fn, 0);
 RCCL_ASSERT_OFFSET(rcclApiFuncTable, ncclAllReduce_fn, 1);
 RCCL_ASSERT_OFFSET(rcclApiFuncTable, ncclAllToAll_fn, 2);
@@ -211,10 +227,15 @@ RCCL_ASSERT_OFFSET(rcclApiFuncTable, mscclRunAlgo_fn, 33);
 RCCL_ASSERT_OFFSET(rcclApiFuncTable, mscclUnloadAlgo_fn, 34);
 RCCL_ASSERT_OFFSET(rcclApiFuncTable, ncclCommRegister_fn, 35);
 RCCL_ASSERT_OFFSET(rcclApiFuncTable, ncclCommDeregister_fn, 36);
+RCCL_ASSERT_OFFSET(rcclApiFuncTable, ncclAllReduceWithBias_fn, 37);
+RCCL_ASSERT_OFFSET(rcclApiFuncTable, ncclCommShrink_fn, 38);
+RCCL_ASSERT_OFFSET(rcclApiFuncTable, ncclCommWindowRegister_fn, 39);
+RCCL_ASSERT_OFFSET(rcclApiFuncTable, ncclCommWindowDeregister_fn, 40);
+// DO NOT REORDER, ADD NEW ITEMS HERE
 
 #undef RCCL_ASSERT_OFFSET
 
-static_assert(sizeof(rcclApiFuncTable) == compute_table_size(37),
+static_assert(sizeof(rcclApiFuncTable) == compute_table_size(41),
               "Update table major/step version and add a new offset assertion if this "
               "fails to compile");
 
@@ -261,7 +282,13 @@ RcclGetFunctionTable_impl()
                                                &mscclRunAlgo_impl,
                                                &mscclUnloadAlgo_impl,
                                                &ncclCommRegister_impl,
-                                               &ncclCommDeregister_impl };
+                                               &ncclCommDeregister_impl,
+                                               &ncclAllReduceWithBias_impl,
+                                               &ncclCommShrink_impl,
+                                               &ncclCommWindowRegister_impl,
+                                               &ncclCommWindowDeregister_impl
+                                               // DO NOT REORDER, ADD NEW ITEMS HERE
+                                             };
 
 #if defined(RCCL_ROCPROFILER_REGISTER) && RCCL_ROCPROFILER_REGISTER > 0
     std::array<void*, 1>                       table_array{ tbl };
@@ -301,6 +328,9 @@ NCCL_API(ncclResult_t, ncclAllGather, const void* sendbuff, void* recvbuff,
 NCCL_API(ncclResult_t, ncclAllReduce, const void* sendbuff, void* recvbuff, size_t count,
          ncclDataType_t datatype, ncclRedOp_t op, ncclComm* comm, hipStream_t stream);
 
+NCCL_API(ncclResult_t, ncclAllReduceWithBias, const void* sendbuff, void* recvbuff, size_t count,
+         ncclDataType_t datatype, ncclRedOp_t op, ncclComm* comm, hipStream_t stream, const void* acc);
+
 NCCL_API(ncclResult_t, ncclAllToAll, const void* sendbuff, void* recvbuff, size_t count,
          ncclDataType_t datatype, ncclComm_t comm, hipStream_t stream);
 
@@ -360,6 +390,9 @@ NCCL_API(ncclResult_t, ncclCommDestroy, ncclComm_t comm);
 
 NCCL_API(ncclResult_t, ncclCommAbort, ncclComm_t comm);
 
+NCCL_API(ncclResult_t, ncclCommShrink, ncclComm_t comm, int* excludeRanksList, int excludeRanksCount,
+         ncclComm_t* newcomm, ncclConfig_t* config, int shrinkFlags);
+
 NCCL_API(ncclResult_t, ncclCommSplit, ncclComm_t comm, int color, int key,
          ncclComm_t* newcomm, ncclConfig_t* config);
 
@@ -395,6 +428,11 @@ NCCL_API(ncclResult_t, ncclCommRegister, const ncclComm_t comm, void* buff, size
 
 NCCL_API(ncclResult_t, ncclCommDeregister, const ncclComm_t comm, void* handle);
 
+NCCL_API(ncclResult_t, ncclCommWindowRegister, ncclComm_t comm, void* buff, size_t size, 
+         ncclWindow_t* win, int winFlags);
+
+NCCL_API(ncclResult_t, ncclCommWindowDeregister, ncclComm_t comm, ncclWindow_t win);
+
 ncclResult_t
 ncclAllGather(const void* sendbuff, void* recvbuff, size_t sendcount,
               ncclDataType_t datatype, ncclComm_t comm, cudaStream_t stream)
@@ -411,6 +449,14 @@ ncclAllReduce(const void* sendbuff, void* recvbuff, size_t count, ncclDataType_t
                                                             datatype, op, comm, stream);
 }
 
+ncclResult_t
+ncclAllReduceWithBias(const void* sendbuff, void* recvbuff, size_t count, ncclDataType_t datatype,
+              ncclRedOp_t op, ncclComm* comm, cudaStream_t stream, const void* acc)
+{
+    return ::rccl::RcclGetFunctionTable()->ncclAllReduceWithBias_fn(sendbuff, recvbuff, count,
+                                                            datatype, op, comm, stream, acc);
+}
+
 ncclResult_t
 ncclAllToAll(const void* sendbuff, void* recvbuff, size_t count, ncclDataType_t datatype,
              ncclComm_t comm, hipStream_t stream)
@@ -563,6 +609,14 @@ ncclCommAbort(ncclComm_t comm)
     return ::rccl::RcclGetFunctionTable()->ncclCommAbort_fn(comm);
 }
 
+ncclResult_t
+ncclCommShrink(ncclComm_t comm, int* excludeRanksList, int excludeRanksCount, ncclComm_t* newcomm,
+               ncclConfig_t* config, int shrinkFlags)
+{
+    return ::rccl::RcclGetFunctionTable()->ncclCommShrink_fn(comm, excludeRanksList, excludeRanksCount, 
+                                                             newcomm, config, shrinkFlags);
+}
+
 ncclResult_t
 ncclCommSplit(ncclComm_t comm, int color, int key, ncclComm_t* newcomm,
               ncclConfig_t* config)
@@ -654,3 +708,15 @@ ncclCommDeregister(const ncclComm_t comm, void* handle)
 {
     return ::rccl::RcclGetFunctionTable()->ncclCommDeregister_fn(comm, handle);
 }
+
+ncclResult_t
+ncclCommWindowRegister(ncclComm_t comm, void* buff, size_t size, ncclWindow_t* win, int winFlags)
+{
+    return ::rccl::RcclGetFunctionTable()->ncclCommWindowRegister_fn(comm, buff, size, win, winFlags);
+}
+
+ncclResult_t
+ncclCommWindowDeregister(ncclComm_t comm, ncclWindow_t win)
+{
+    return ::rccl::RcclGetFunctionTable()->ncclCommWindowDeregister_fn(comm, win);
+}
