@@ -8,7 +8,6 @@
 #include "core.h"
 
 #include "nvmlwrap.h"
-#include "archinfo.h"
 
 #include <unistd.h>
 #include <stdlib.h>
@@ -193,18 +192,6 @@ bool matchIfList(const char* string, int port, struct netIf* ifList, int listSiz
   return false;
 }
 
-RCCL_PARAM(DisableContextTracking, "DISABLE_CONTEXT_TRACKING", 0);
-bool rcclNeedEnableContextTrack(int cuDeviceId) {
-  hipDeviceProp_t devProp;
-  if (rcclParamDisableContextTracking() == 1)
-    return false;
-  if (hipGetDeviceProperties(&devProp, cuDeviceId) != 0)
-    return false;
-  return IsArchMatch(devProp.gcnArchName,"gfx11")
-         || IsArchMatch(devProp.gcnArchName,"gfx12")
-         || IsArchMatch(devProp.gcnArchName,"gfx10");
-}
-
 __thread struct ncclThreadSignal ncclThreadSignalLocalInstance = ncclThreadSignalStaticInitializer();
 
 void* ncclMemoryStack::allocateSpilled(struct ncclMemoryStack* me, size_t size, size_t align) {
