@@ -487,6 +487,13 @@ void Replayer::replay()
       NCCL_CALL(ncclAllReduce(sbuffer, rbuffer, call.count, call.datatype, call.op, commMap[call.comm], streams[call.stream].first));
       break;
     }
+    case rrAllReduceWithBias:
+    {
+      std::vector<char> acc(call.count * ncclTypeSize(call.datatype));
+      NCCL_CALL(ncclAllReduceWithBias(sbuffer, rbuffer, call.count, call.datatype, call.op, commMap[call.comm], streams[call.stream].first, acc.data()));
+      HIP_CALL(hipStreamSynchronize(streams[call.stream].first)); // TODO: remove, and further verify behavior of fused AR
+      break;
+    }
     // a2av
     case rrAllToAllv:
     {
@@ -670,4 +677,4 @@ int main(int argc, char **argv)
   replayer.replay();
   MPI_Finalize();
   return 0;
-}
\ No newline at end of file
+}
